'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var ramda = require('ramda');
var reactNative = require('react-native');
var React = require('react');
var React__default = _interopDefault(React);
var reactotronCoreClient = require('reactotron-core-client');
var getHost = _interopDefault(require('rn-host-detect'));

/**
 * Provides a global error handler to report errors..
 */
// a few functions to help source map errors -- these seem to be not available immediately
// so we're lazy loading.
var parseErrorStack = void 0;
var symbolicateStackTrace = void 0;

// defaults
var PLUGIN_DEFAULTS = {
  veto: null // frame -> boolean
};

// const reactNativeFrameFinder = frame => contains('/node_modules/react-native/', frame.fileName)

// our plugin entry point
var trackGlobalErrors = (function (options) {
  return function (reactotron) {
    // setup configuration
    var config = ramda.merge(PLUGIN_DEFAULTS, options || {});

    var swizzled = null;
    var isSwizzled = false;

    function reactotronExceptionHijacker(message, prettyStack, currentExceptionID) {
      // do Facebook's stuff first
      swizzled(message, prettyStack, currentExceptionID);

      // then convert & transport it
      try {
        // rewrite the stack frames to be in the format we're expecting
        var stack = ramda.map(function (frame) {
          return {
            functionName: frame.methodName === '<unknown>' ? null : frame.methodName,
            lineNumber: frame.lineNumber,
            columnNumber: frame.column,
            fileName: frame.file
          };
        }, prettyStack);

        // does the dev want us to keep each frame?
        if (config.veto) {
          stack = ramda.reject(config.veto, stack);
        }

        // throw it over to us
        reactotron.error(message, stack);
      } catch (e) {
        // TODO: no one must ever know our dark secrets
      }
    }

    // here's how to swizzle
    function trackGlobalErrors() {
      if (isSwizzled) return;
      swizzled = reactNative.NativeModules.ExceptionsManager.updateExceptionMessage;
      reactNative.NativeModules.ExceptionsManager.updateExceptionMessage = reactotronExceptionHijacker;
      isSwizzled = true;
    }

    // restore the original
    function untrackGlobalErrors() {
      if (!swizzled) return;
      reactNative.NativeModules.ExceptionsManager.updateExceptionMessage = swizzled;
      isSwizzled = false;
    }

    // auto start this
    trackGlobalErrors();

    // manually fire an error
    function reportError(error) {
      try {
        parseErrorStack = parseErrorStack || require('react-native/Libraries/Core/Devtools/parseErrorStack');
        symbolicateStackTrace = symbolicateStackTrace || require('react-native/Libraries/Core/Devtools/symbolicateStackTrace');
        if (parseErrorStack && symbolicateStackTrace) {
          var parsedStacktrace = parseErrorStack(error);

          symbolicateStackTrace(parsedStacktrace).then(function (goodStack) {
            var stack = goodStack.map(function (stackFrame) {
              return {
                fileName: stackFrame.file,
                functionName: stackFrame.methodName,
                lineNumber: stackFrame.lineNumber
              };
            });

            // does the dev want us to keep each frame?
            if (config.veto) {
              stack = ramda.reject(config.veto, stack);
            }

            reactotron.error(error.message, stack);
          });

          return;
        }
      } catch (e) {
        // nothing happened. move along.
      }
    }

    // the reactotron plugin interface
    return {
      // attach these functions to the Reactotron
      features: {
        reportError: reportError,
        trackGlobalErrors: trackGlobalErrors,
        untrackGlobalErrors: untrackGlobalErrors
      }
    };
  };
});

var DEFAULTS$1 = {
  url: 'http://localhost:8081'
};

var openInEditor = (function () {
  var pluginConfig = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return function (reactotron) {
    var options = ramda.merge(DEFAULTS$1, pluginConfig);

    return {
      onCommand: function onCommand(command) {
        if (command.type !== 'editor.open') return;
        var payload = command.payload;
        var file = payload.file,
            lineNumber = payload.lineNumber;

        var url = options.url + '/open-stack-frame';
        var body = { file: file, lineNumber: lineNumber || 1 };
        var method = 'POST';

        fetch(url, { method: method, body: JSON.stringify(body) });
      }
    };
  };
});

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();







var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};



var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};











var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var Styles = {
  container: {
    position: 'absolute',
    left: 0,
    top: 0,
    right: 0,
    bottom: 0,
    zIndex: 1000,
    opacity: 0.25
  }
};

/** An overlay for showing an image to help with layout.
 *
 * @class FullScreenOverlay
 * @extends {Component}
 */

var FullScreenOverlay = function (_Component) {
  inherits(FullScreenOverlay, _Component);

  /**
   * Creates an instance of FullScreenOverlay.
   *
   * @param {any} props
   * @param {Object} props.emitter An event emitter.
   *
   * @memberOf FullScreenOverlay
   */
  function FullScreenOverlay(props) {
    classCallCheck(this, FullScreenOverlay);

    var _this = possibleConstructorReturn(this, (FullScreenOverlay.__proto__ || Object.getPrototypeOf(FullScreenOverlay)).call(this, props));

    _this.state = {
      opacity: Styles.container.opacity,
      uri: null,
      justifyContent: 'center',
      alignItems: 'center'
    };

    // when the server sends stuff
    props.emitter.on('overlay', function (payload) {
      _this.setState(_extends({}, _this.state, payload));
    });
    return _this;
  }

  /**
   * Makes the styles used by the root container.
   *
   * @returns {Object} The styles.
   *
   * @memberOf FullScreenOverlay
   */


  /** The types of the properties. */


  createClass(FullScreenOverlay, [{
    key: 'createContainerStyle',
    value: function createContainerStyle() {
      var _state = this.state,
          opacity = _state.opacity,
          justifyContent = _state.justifyContent,
          alignItems = _state.alignItems;

      var _Dimensions$get = reactNative.Dimensions.get('window'),
          width = _Dimensions$get.width,
          height = _Dimensions$get.height;

      var containerStyle = _extends({}, Styles.container, { opacity: opacity, width: width, height: height, justifyContent: justifyContent, alignItems: alignItems });

      return containerStyle;
    }

    /**
     * Draw.
     */

  }, {
    key: 'render',
    value: function render() {
      var _state2 = this.state,
          uri = _state2.uri,
          width = _state2.width,
          height = _state2.height,
          growToWindow = _state2.growToWindow,
          resizeMode = _state2.resizeMode,
          _state2$marginLeft = _state2.marginLeft,
          marginLeft = _state2$marginLeft === undefined ? 0 : _state2$marginLeft,
          _state2$marginRight = _state2.marginRight,
          marginRight = _state2$marginRight === undefined ? 0 : _state2$marginRight,
          _state2$marginTop = _state2.marginTop,
          marginTop = _state2$marginTop === undefined ? 0 : _state2$marginTop,
          _state2$marginBottom = _state2.marginBottom,
          marginBottom = _state2$marginBottom === undefined ? 0 : _state2$marginBottom;

      var imageStyle = { width: width, height: height, marginTop: marginTop, marginRight: marginRight, marginBottom: marginBottom, marginLeft: marginLeft };
      if (growToWindow) {
        var windowSize = reactNative.Dimensions.get('window');
        imageStyle.width = windowSize.width;
        imageStyle.height = windowSize.height;
      }
      var image = uri ? React__default.createElement(reactNative.Image, { source: { uri: uri }, style: imageStyle, resizeMode: growToWindow ? resizeMode : null }) : React__default.createElement(reactNative.View, null);

      return React__default.createElement(
        reactNative.View,
        {
          style: this.createContainerStyle(),
          pointerEvents: 'none'
        },
        image
      );
    }
  }]);
  return FullScreenOverlay;
}(React.Component);

FullScreenOverlay.propTypes = {
  /**
   * An emitter which can be subscribed to to listen for events.
   */
  emitter: React.PropTypes.object.isRequired
};

/** Mitt: Tiny (~200b) functional event emitter / pubsub.
 *
 * NOTE: https://github.com/developit/mitt/pull/24
 *
 *  @name mitt
 *  @returns {Mitt}
 */
function mitt(all) {
  // Arrays of event handlers, keyed by type
  all = all || {};

  // Get or create a named handler list
  function list(type) {
    var t = type.toLowerCase();
    return all[t] || (all[t] = []);
  }

  return {

    /** Register an event handler for the given type.
     *  @param {String} type  Type of event to listen for, or `"*"` for all events
     *  @param {Function} handler  Function to call in response to the given event
     *  @memberof mitt
     */
    on: function on(type, handler) {
      list(type).push(handler);
    },


    /** Remove an event handler for the given type.
     *  @param {String} type  Type of event to unregister `handler` from, or `"*"`
     *  @param {Function} handler  Handler function to remove
     *  @memberof mitt
     */
    off: function off(type, handler) {
      var e = list(type);
      var i = e.indexOf(handler);
      if (~i) e.splice(i, 1);
    },


    /** Invoke all handlers for the given type.
     *  If present, `"*"` handlers are invoked prior to type-matched handlers.
     *  @param {String} type  The event type to invoke
     *  @param {Any} [event]  An event object, passed to each handler
     *  @memberof mitt
     */
    emit: function emit(type, event) {
      list('*').concat(list(type)).forEach(function (f) {
        f(event);
      });
    }
  };
}

/**
 * Provides an image.
 */
var overlay = (function () {
  return function (reactotron) {
    var emitter = mitt();

    return {
      /**
       * Fires when any Reactotron message arrives.
       *
       * @param {object} command The Reactotron command object.
       */
      onCommand: function onCommand(command) {
        if (command.type !== 'overlay') return;
        // relay this payload on to the emitter
        emitter.emit('overlay', command.payload);
      },

      features: {
        overlay: function overlay(WrappedComponent) {
          return function (props) {
            return React__default.createElement(
              reactNative.View,
              { style: { flex: 1 } },
              React__default.createElement(WrappedComponent, null),
              React__default.createElement(FullScreenOverlay, { emitter: emitter })
            );
          };
        }
      }
    };
  };
});

// -----------
// FIRST PARTY
// -----------

// ------------
// SECOND PARTY
// ------------

// -----------
// THIRD PARTY
// -----------

var io = require('socket.io-client/dist/socket.io');

// ---------------------
// DEFAULT CONFIGURATION
// ---------------------

var DEFAULTS = {
  io: io,
  host: getHost('localhost'),
  port: 9090,
  name: 'React Native App',
  userAgent: 'reactotron-react-native',
  reactotronVersion: 'BETA', // TODO: figure this out for realz.  why is this hard?  it must be me.
  environment: __DEV__ ? 'development' : 'production' // naive... TODO: find the right way ya lazy bum.
};

// -----------
// HERE WE GO!
// -----------
// Create the default reactotron.
var reactotron = reactotronCoreClient.createClient(DEFAULTS);

exports.trackGlobalErrors = trackGlobalErrors;
exports.openInEditor = openInEditor;
exports.overlay = overlay;
exports['default'] = reactotron;
